/* (C) 2021 YiRing, Inc. */
package com.yiring.common.param;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serial;
import java.io.Serializable;
import java.util.Objects;
import javax.validation.constraints.DecimalMin;
import javax.validation.constraints.NotNull;
import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.FieldDefaults;
import lombok.experimental.SuperBuilder;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;

/**
 * 公共的分页排序查询参数
 *
 * @author ifzm
 * @version 0.1 2019/3/10 16:29
 */
@ApiModel(value = "PageParam", description = "公共的分页排序查询参数")
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@FieldDefaults(level = AccessLevel.PRIVATE)
public class PageParam implements Serializable {

    @Serial
    private static final long serialVersionUID = 6103761701912769946L;

    @ApiModelProperty(value = "分页条数", example = "10", required = true)
    @NotNull(message = "分页条数不能为空")
    @DecimalMin(value = "1", message = "分页条数不能小于1")
    Integer pageSize;

    @ApiModelProperty(value = "当前页数", example = "1", required = true)
    @NotNull(message = "当前页数不能为空")
    @DecimalMin(value = "1", message = "当前页数不能小于1")
    Integer pageNo;

    @ApiModelProperty(value = "排序字段", example = "id")
    String sortField;

    @ApiModelProperty(value = "排序方向（ASC|DESC）", example = "DESC")
    Sort.Direction sortOrder;

    /**
     * 根据参数构建分页对象
     *
     * @param param {@link PageParam}
     * @return Pageable
     */
    @SuppressWarnings({ "unused" })
    public static Pageable toPageable(PageParam param) {
        if (param == null) {
            return Pageable.unpaged();
        }

        return PageParam.toPageable(param.getSortField(), param.getSortOrder(), param.pageSize, param.getPageNo());
    }

    /**
     * 根据参数构建分页对象
     *
     * @param sortField 排序字段
     * @param sortOrder 排序方向
     * @param pageSize 分页大小
     * @param pageNo 分页页码
     * @return Pageable
     */
    public static Pageable toPageable(String sortField, Sort.Direction sortOrder, Integer pageSize, Integer pageNo) {
        Sort sort = Sort.unsorted();
        if (Objects.nonNull(sortField)) {
            sort = Sort.by(new Sort.Order(sortOrder, sortField));
        }
        return PageRequest.of(pageSize - 1, pageNo, sort);
    }
}
